@@ -48,11 +48,14 @@ struct symresolution_info: public traversing_visitor
 {
 protected:
   systemtap_session& session;
+  symresolution_info* saved_session_symbol_resolver;
+  
   bool unmangled_p;
 public:
   functiondecl* current_function;
-  derived_probe* current_probe;
+  probe* current_probe;
   symresolution_info (systemtap_session& s, bool omniscient_unmangled = false);
+  ~symresolution_info ();
 
   vardecl* find_var (const std::string& name, int arity, const token *tok);
   std::vector<functiondecl*> find_functions (functioncall *call, const std::string& name, unsigned arity, const token *tok);
@@ -111,7 +114,9 @@ struct typeresolution_info: public visitor
                  const symboldecl* decl, int index = -1);
   void resolved (const token* tok, exp_type type,
                  const symboldecl* decl = NULL, int index = -1);
-  void resolved_details (const exp_type_ptr& src, exp_type_ptr& dest);
+  void resolve_details (const token* tok, const exp_type_ptr& src, exp_type_ptr& dest);
+  void resolve(exp_type t, expression* e);
+  void resolve(exp_type t, symboldecl* d);
 
   exp_type t; // implicit parameter for nested visit call; may clobber
               // Upon entry to one of the visit_* calls, the incoming
@@ -170,6 +175,7 @@ struct typeresolution_info: public visitor
   void visit_autocast_op (autocast_op* e);
   void visit_atvar_op (atvar_op* e);
   void visit_defined_op (defined_op* e);
+  void visit_probewrite_op(probewrite_op* e);
   void visit_entry_op (entry_op* e);
   void visit_perf_op (perf_op* e);
 
@@ -199,12 +205,13 @@ struct derived_probe: public probe
   virtual void join_group (systemtap_session& s) = 0;
   virtual probe_point* sole_location () const;
   virtual probe_point* script_location () const;
-  virtual void printsig (std::ostream &o, bool nest=true) const;
+  virtual void printsig (std::ostream &o) const cxx_override;
+  virtual void printsig_nonest (std::ostream &o) const;
   // return arguments of probe if there
   virtual void getargs (std::list<std::string> &) const {}
   void printsig_nested (std::ostream &o) const;
-  virtual void collect_derivation_chain (std::vector<probe*> &probes_list) const;
-  virtual void collect_derivation_pp_chain (std::vector<probe_point*> &pp_list) const;
+  virtual void collect_derivation_chain (std::vector<probe*> &probes_list) const cxx_override;
+  virtual void collect_derivation_pp_chain (std::vector<probe_point*> &pp_list) const cxx_override;
   std::string derived_locations (bool firstFrom = true);
 
   virtual void print_dupe_stamp(std::ostream&) {}
@@ -386,6 +393,7 @@ match_key
 };
 
 
+class lsp_method_text_document_completion; // Used for the language-server
 class
 match_node
 {
@@ -417,6 +425,8 @@ match_node
 
 private:
   privilege_t privilege;
+
+  friend class lsp_method_text_document_completion;
 };
 
 // ------------------------------------------------------------------------
@@ -466,6 +476,18 @@ symbol * get_symbol_within_expression (expression *e);
 struct unparser;
 
 
+struct probewrite_evaluator: public update_visitor
+{
+  systemtap_session& session;
+  statement* probe_body;
+
+  probewrite_evaluator(systemtap_session& s, statement* probe_body):
+    session(s), probe_body(probe_body) {}
+
+  void visit_probewrite_op(probewrite_op* e);
+};
+
+
 struct const_folder: public update_visitor
 {
   systemtap_session& session;
@@ -505,8 +527,6 @@ struct const_folder: public update_visitor
   void visit_target_symbol (target_symbol* e);
 };
 
-
-
 // Run the given code filter visitors against the given body.
 // Repeat until they all report having relaxed.
 template <class T>
@@ -532,7 +552,7 @@ void update_visitor_loop (systemtap_session& sess, std::vector<update_visitor*>&
 template <class X, class Y>
 void var_expand_const_fold_loop(systemtap_session& sess, X& body, Y& v)
 {
-  bool relaxed_p; /* ignored */
+  bool relaxed_p = false;
   const_folder cf (sess, relaxed_p);
   std::vector<update_visitor*> k;
   k.push_back (& v);
