@@ -60,6 +60,7 @@ extern "C" {
 #include <wordexp.h>
 #include <ftw.h>
 #include <fcntl.h>
+#include <sys/wait.h>
 }
 
 using namespace std;
@@ -1371,10 +1372,25 @@ passes_0_4 (systemtap_session &s)
   return rc;
 }
  
+// Unprivileged (PR30321) part of pass 5
 int
-pass_5 (systemtap_session &s, vector<remote*> targets)
+pass_5_1 (systemtap_session &s, vector<remote*> targets)
 {
-  // PASS 5: RUN
+  // PASS 5: RUN (part 1 - unprivileged)
+  s.verbose = s.perpass_verbose[4];
+  int rc;
+  // Prepare the staprun cmdline, but don't spawn it.
+  // Store staprun cmdline in s->tmpdir + "/staprun_args".
+  // Run under unprivileged user.
+  rc = remote::run1(targets);
+  return rc;
+}
+
+// Privileged (PR30321) part of pass 5
+int
+pass_5_2 (systemtap_session &s, vector<remote*> targets)
+{
+  // PASS 5: RUN (part 2 - privileged)
   s.verbose = s.perpass_verbose[4];
   struct tms tms_before;
   times (& tms_before);
@@ -1385,7 +1401,10 @@ pass_5 (systemtap_session &s, vector<remote*> targets)
   // and don't take an indefinite amount of time.
   PROBE1(stap, pass5__start, &s);
   if (s.verbose) clog << _("Pass 5: starting run.") << endl;
-  int rc = remote::run(targets);
+  // Spawn staprun with already prepared commandline.
+  // Retreive staprun cmdline in s->tmpdir + "/staprun_args".
+  // Run under privileged user.
+  int rc = remote::run2(targets);
   struct tms tms_after;
   times (& tms_after);
   unsigned _sc_clk_tck = sysconf (_SC_CLK_TCK);
@@ -1451,6 +1470,176 @@ passes_0_4_again_with_server (systemtap_session &s)
   return rc;
 }
 
+static int
+passes_1_5 (systemtap_session &s, vector<remote*> targets)
+{
+  int rc = 0;
+
+  // Discover and loop over each unique session created by the remote targets.
+  set<systemtap_session*> sessions;
+  for (unsigned i = 0; i < targets.size(); ++i)
+    sessions.insert(targets[i]->get_session());
+
+  for (set<systemtap_session*>::iterator it = sessions.begin();
+       rc == 0 && !pending_interrupts && it != sessions.end(); ++it)
+    {
+      systemtap_session& ss = **it;
+      if (ss.verbose > 1)
+        clog << _F("Session arch: %s release: %s",
+                   ss.architecture.c_str(), ss.kernel_release.c_str())
+             << endl
+             << _F("Build tree: \"%s\"",
+                   ss.kernel_build_tree.c_str())
+             << endl;
+
+#if HAVE_NSS
+      // If requested, query server status. This is independent
+      // of other tasks.
+      nss_client_query_server_status (ss);
+
+      // If requested, manage trust of servers. This is
+      // independent of other tasks.
+      nss_client_manage_server_trust (ss);
+#endif
+  
+      // Run the passes only if a script has been specified or
+      // if we're dumping something. The requirement for a
+      // script has already been checked in
+      // systemtap_session::check_options.
+      if (ss.have_script || ss.dump_mode)
+        {
+          // Run passes 0-4 for each unique session, either
+          // locally or using a compile-server.
+          ss.init_try_server ();
+          if ((rc = passes_0_4 (ss)))
+            {
+              // Compilation failed.
+              // Try again using a server if appropriate.
+              if (ss.try_server ())
+                rc = passes_0_4_again_with_server (ss);
+            }
+          if (rc || s.perpass_verbose[0] >= 1)
+            s.explain_auto_options ();
+        }
+    }
+  
+  if (rc == 0 && s.have_script && s.last_pass >= 5 && ! pending_interrupts)
+    {
+      rc = pass_5_1 (s, targets);
+      rc = pass_5_2 (s, targets);
+    }
+  return rc;
+}
+
+
+
+static int
+passes_1_5_build_as (systemtap_session &s, vector<remote*> targets)
+{
+  int rc = 0;
+  // logging verbosity treshold
+  unsigned int vt = 2;
+
+  // Discover and loop over each unique session created by the remote targets.
+  set<systemtap_session*> sessions;
+  for (unsigned i = 0; i < targets.size(); ++i)
+    sessions.insert(targets[i]->get_session());
+
+  // PR30321: Privilege separation
+  // Fork the stap process in two:  An unprivileged child, and a privileged parent
+  // Child will run passes 1-4 and part of pass 5 (up to preparing staprun cmdline
+  // Parent will wait, spawn staprun (second part of pass 5), and finish.
+  pid_t frkrc = fork();
+  if (frkrc == -1)
+    {
+      clog << _("ERROR: Fork failed.  Terminating...") << endl;
+      return EXIT_FAILURE;
+    }
+  else if (frkrc == 0)
+    {
+      // Child process
+      if (s.build_as != "")
+        {
+          // Start running under an unprivileged user
+          rc = run_unprivileged(s.build_as, s.build_as_uid, s.build_as_gid, s.verbose);
+          if (rc != EXIT_SUCCESS)
+              return rc;
+
+          if (s.verbose >= vt)
+            clog << _F("Child pid=%d, uid=%d, euid=%d, gid=%d, egid=%d\n",
+                       getpid(), getuid(), geteuid(), getgid(), getegid());
+        }
+
+      for (set<systemtap_session*>::iterator it = sessions.begin();
+           rc == 0 && !pending_interrupts && it != sessions.end(); ++it)
+        {
+          systemtap_session& ss = **it;
+          if (ss.verbose > 1)
+            clog << _F("Session arch: %s release: %s",
+                       ss.architecture.c_str(), ss.kernel_release.c_str())
+                 << endl
+                 << _F("Build tree: \"%s\"", ss.kernel_build_tree.c_str())
+                 << endl;
+
+#if HAVE_NSS
+          // If requested, query server status. This is independent
+          // of other tasks.
+          nss_client_query_server_status (ss);
+
+          // If requested, manage trust of servers. This is
+          // independent of other tasks.
+          nss_client_manage_server_trust (ss);
+#endif
+
+          // Run the passes only if a script has been specified or
+          // if we're dumping something. The requirement for a
+          // script has already been checked in
+          // systemtap_session::check_options.
+          if (ss.have_script || ss.dump_mode)
+            {
+              // Run passes 0-4 for each unique session, either
+              // locally or using a compile-server.
+              ss.init_try_server ();
+              if ((rc = passes_0_4 (ss)))
+                {
+                  // Compilation failed.
+                  // Try again using a server if appropriate.
+                  if (ss.try_server ())
+                    rc = passes_0_4_again_with_server (ss);
+                }
+              if (rc || s.perpass_verbose[0] >= 1)
+                s.explain_auto_options ();
+            }
+        }
+
+      // Run pass 5, if requested (part 1/2 (unprivileged))
+      if (rc == 0 && s.have_script && s.last_pass >= 5 && ! pending_interrupts)
+        rc = pass_5_1 (s, targets);
+      if (s.verbose >= vt)
+        clog << _F("Child finished running, tmpdir is %s", s.tmpdir.c_str())
+             << endl;
+      _exit(rc);
+    }
+  else
+    {
+      // Parent process
+      if (s.verbose >= vt)
+        clog << _F("Parent pid=%d, uid=%d, euid=%d, gid=%d, egid=%d\n",
+                   getpid(), getuid(), geteuid(), getgid(), getegid());
+      int wstatus;
+      (void)waitpid(frkrc, &wstatus, 0);
+      rc = WEXITSTATUS(wstatus);
+    }
+
+  // Run pass 5, if requested (part 2/2 (privileged))
+  if (s.verbose >= vt)
+    clog << _F("Parent about to execute staprun, tmpdir is %s", s.tmpdir.c_str())
+         << endl;
+  if (rc == 0 && s.have_script && s.last_pass >= 5 && ! pending_interrupts)
+    rc = pass_5_2 (s, targets);
+  return rc;
+}
+
 int
 main (int argc, char * const argv [])
 {
@@ -1562,10 +1751,6 @@ main (int argc, char * const argv [])
           rc = 1;
       }
 
-    // Discover and loop over each unique session created by the remote targets.
-    set<systemtap_session*> sessions;
-    for (unsigned i = 0; i < targets.size(); ++i)
-      sessions.insert(targets[i]->get_session());
 
     #if HAVE_LANGUAGE_SERVER_SUPPORT
     if(s.language_server_mode){
@@ -1584,52 +1769,10 @@ main (int argc, char * const argv [])
       }
     else
       {
-	for (set<systemtap_session*>::iterator it = sessions.begin();
-	     rc == 0 && !pending_interrupts && it != sessions.end(); ++it)
-	  {
-	    systemtap_session& ss = **it;
-            if (ss.verbose > 1)
-	      clog << _F("Session arch: %s release: %s",
-			 ss.architecture.c_str(), ss.kernel_release.c_str())
-		   << endl
-                   << _F("Build tree: \"%s\"",
-			 ss.kernel_build_tree.c_str())
-		   << endl;
-
-#if HAVE_NSS
-	    // If requested, query server status. This is independent
-	    // of other tasks.
-	    nss_client_query_server_status (ss);
-
-	    // If requested, manage trust of servers. This is
-	    // independent of other tasks.
-	    nss_client_manage_server_trust (ss);
-#endif
-
-	    // Run the passes only if a script has been specified or
-	    // if we're dumping something. The requirement for a
-	    // script has already been checked in
-	    // systemtap_session::check_options.
-	    if (ss.have_script || ss.dump_mode)
-	      {
-		// Run passes 0-4 for each unique session, either
-		// locally or using a compile-server.
-		ss.init_try_server ();
-		if ((rc = passes_0_4 (ss)))
-		  {
-		    // Compilation failed.
-		    // Try again using a server if appropriate.
-		    if (ss.try_server ())
-		      rc = passes_0_4_again_with_server (ss);
-		  }
-		if (rc || s.perpass_verbose[0] >= 1)
-		  s.explain_auto_options ();
-	      }
-	  }
-
-	// Run pass 5, if requested
-	if (rc == 0 && s.have_script && s.last_pass >= 5 && ! pending_interrupts)
-	  rc = pass_5 (s, targets);
+        if (s.build_as != "")
+          rc = passes_1_5_build_as(s, targets);
+        else
+          rc = passes_1_5(s, targets);
       }
 
     // Pass 6. Cleanup
