@@ -1,5 +1,5 @@
 /* target operations in the Dyninst mode
- * Copyright (C) 2012 Red Hat Inc.
+ * Copyright (C) 2012, 2016 Red Hat Inc.
  *
  * This file is part of systemtap, and is free software.  You can
  * redistribute it and/or modify it under the terms of the GNU General
@@ -31,6 +31,9 @@
 #endif
 
 
+#define _stp_deref_nofault(value, size, addr, seg)			\
+    __copy_from_user((void *)&(value), (void *)(addr), (size_t)(size))
+
 #define uread(ptr) ({ \
 	typeof(*(ptr)) _v = 0; \
 	if (__copy_from_user((void *)&_v, (void *)(ptr), sizeof(*(ptr)))) \
@@ -74,5 +77,9 @@
 #define kderef uderef
 #define store_kderef store_uderef
 
+#define stp_mem_access_begin(type, ptr, size, oldfs, seg, is_user_ptr)
+#define stp_mem_access_end(oldfs, is_user)
+#define stp_user_access_begin(type, ptr, size, oldfs, seg)
+#define stp_user_access_end(oldfs)
 
 #endif /* _STAPDYN_LOC2C_RUNTIME_H_ */
