@@ -1,6 +1,8 @@
 #include <linux/stacktrace.h>
 
-unsigned int foo ()
+unsigned int foo (void);
+
+unsigned int foo (void)
 {
         unsigned long e[10];
         struct pt_regs* r = 0;
