@@ -20,7 +20,12 @@
  * @param ri Pointer to the struct kretprobe_instance.
  * @return The return address
  */
-#define _stp_ret_addr_r(ri) (ri->ret_addr)
+// PR29028, ret_addr field may be located in a another struct
+#ifdef CONFIG_KRETPROBE_ON_RETHOOK
+#define _stp_ret_addr_r(ri) ((ri)->node.ret_addr)
+#else
+#define _stp_ret_addr_r(ri) ((ri)->ret_addr)
+#endif
 
 /** Get the probe address for a kprobe.
  * Call from a kprobe. This will return the
@@ -36,7 +41,7 @@
  * @param ri Pointer to the struct kretprobe_instance.
  * @return The function's address
  */
-#define _stp_probe_addr_r(ri) (ri->rp->kp.addr)
+#define _stp_probe_addr_r(ri) (get_kretprobe(ri)->kp.addr)
 
 #if defined  (__x86_64__)
 
@@ -182,6 +187,10 @@ static void _stp_print_regs(struct pt_regs * regs)
 		}
 
 		_stp_printf("%016lX ", regs->gpr[i]);
+/* since kernel commit 8dc7f0229 */
+#ifndef FULL_REGS
+#define FULL_REGS(r) true
+#endif
 		if (i == 13 && !FULL_REGS(regs))
 			break;
 	}
@@ -346,7 +355,7 @@ static int _stp_probing_app_with_32bit_regs(struct pt_regs *regs)
 	if (!regs)
 		return 0;
 	return (user_mode(regs) &&
-			test_tsk_thread_flag(current, TIF_32BIT_REGS));
+                _stp_is_compat_task());
 }
 
 void _stp_print_regs(struct pt_regs * regs)
