@@ -1,10 +1,10 @@
 /* -*- linux-c -*- 
  *
- * This transport version uses relayfs on top of a debugfs file.  This
- * code started as a proposed relayfs interface called 'utt'.  It has
- * been modified and simplified for systemtap.
+ * This transport version uses relayfs on top of a debugfs or procfs
+ * file.  This code started as a proposed relayfs interface called
+ * 'utt'.  It has been modified and simplified for systemtap.
  *
- * Changes Copyright (C) 2009-2014 Red Hat Inc.
+ * Changes Copyright (C) 2009-2020 Red Hat Inc.
  *
  * Original utt code by:
  *   Copyright (C) 2006 Jens Axboe <axboe@suse.de>
@@ -29,14 +29,13 @@
 #include <linux/init.h>
 #include <linux/debugfs.h>
 #include <linux/mm.h>
-#include <linux/relay.h>
 #include "../linux/timer_compatibility.h"
 #include "../uidgid_compatibility.h"
 #include "relay_compat.h"
 
 #ifndef STP_RELAY_TIMER_INTERVAL
-/* Wakeup timer interval in jiffies (default 10 ms) */
-#define STP_RELAY_TIMER_INTERVAL		((HZ + 99) / 100)
+/* Wakeup timer interval in jiffies (default 1 ms) */
+#define STP_RELAY_TIMER_INTERVAL		((HZ + 999) / 1000)
 #endif
 
 /* Note: if struct _stp_relay_data_type changes, staplog.c might need
@@ -44,10 +43,6 @@
 struct _stp_relay_data_type {
 	struct rchan *rchan;
 	atomic_t /* enum _stp_transport_state */ transport_state;
-#ifdef _STP_USE_DROPPED_FILE
-	struct dentry *dropped_file;
-	atomic_t dropped;
-#endif
 	atomic_t wakeup;
 	struct timer_list timer;
 	int overwrite_flag;
@@ -72,7 +67,7 @@ static size_t __stp_relay_switch_subbuf(struct rchan_buf *buf, size_t length)
 		return 0;
 
 	if (unlikely(length > buf->chan->subbuf_size))
-		goto toobig;
+		length = buf->chan->subbuf_size;
 
 	if (buf->offset != buf->chan->subbuf_size + 1) {
 		buf->prev_padding = buf->chan->subbuf_size - buf->offset;
@@ -82,14 +77,6 @@ static size_t __stp_relay_switch_subbuf(struct rchan_buf *buf, size_t length)
 		buf->dentry->d_inode->i_size += buf->chan->subbuf_size -
 			buf->padding[old_subbuf];
 		smp_mb();
-		if (waitqueue_active(&buf->read_wait))
-			/*
-			 * Calling wake_up_interruptible() and __mod_timer()
-			 * from here will deadlock if we happen to be logging
-			 * from the scheduler and timer (trying to re-grab
-			 * rq->lock/timer->base->lock), so just set a flag.
-			 */
-			atomic_set(&_stp_relay_data.wakeup, 1);
 	}
 
 	old = buf->data;
@@ -103,43 +90,103 @@ static size_t __stp_relay_switch_subbuf(struct rchan_buf *buf, size_t length)
 	buf->data = new;
 	buf->padding[new_subbuf] = 0;
 
-	if (unlikely(length + buf->offset > buf->chan->subbuf_size))
-		goto toobig;
-
 	return length;
+}
+
+static bool __stp_relay_buf_empty(struct rchan_buf *buf)
+{
+	return !buf->offset && buf->subbufs_produced == buf->subbufs_consumed;
+}
+
+static int __stp_relay_file_open(struct inode *inode, struct file *filp)
+{
+	struct rchan_buf *buf = inode->i_private;
+
+	/* kernel's relay_file_open() does not check NULL buf (at least as of
+	 * Sep 9, 2022) and there is a race window in our
+	 * __stp_procfs_relay_create_buf_file_callback() function,
+	 * between proc_create() and the `in->i_private = buf` assignment.
+	 * without this check, we can reproduce a NULL ptr deref kernel panic
+	 * while loading a system with a lot of short-lived stap sessions.
+	 */
+	if (unlikely(buf == NULL)) {
+		dbug_trans(0, "relay_file_open: found NULL rchan buf\n");
+		return -ENOENT;
+	}
+
+	return relay_file_operations.open(inode, filp);
+}
+
+static unsigned int __stp_relay_file_poll(struct file *filp, poll_table *wait)
+{
+	struct rchan_buf *buf = filp->private_data;
+
+	if (buf->finalized)
+		return POLLERR;
+
+	if (filp->f_mode & FMODE_READ) {
+		poll_wait(filp, &buf->read_wait, wait);
+		if (!__stp_relay_buf_empty(buf))
+			return POLLIN | POLLRDNORM;
+	}
 
-toobig:
-	buf->chan->last_toobig = length;
 	return 0;
 }
 
+static ssize_t __stp_relay_file_read(struct file *filp, char __user *buffer,
+				     size_t count, loff_t *ppos)
+{
+	struct rchan_buf *buf = filp->private_data;
+	unsigned long flags;
+
+	/* This trylock will only fail after the print driver is destroyed */
+	if (_stp_print_trylock_irqsave(&flags)) {
+		/* Switch out the current buffer if it has any data */
+		if (buf->offset)
+			__stp_relay_switch_subbuf(buf, 0);
+		_stp_print_unlock_irqrestore(&flags);
+	}
+
+	/*
+	 * Optimization: bail out early if there's nothing to read. This is
+	 * faster than going into relay's read() function without having any
+	 * data, as it will acquire a mutex lock for the inode before bailing.
+	 */
+	if (buf->subbufs_produced == buf->subbufs_consumed)
+		return -ENODATA;
+
+	return relay_file_operations.read(filp, buffer, count, ppos);
+}
+
 static void __stp_relay_wakeup_readers(struct rchan_buf *buf)
 {
 	if (buf && waitqueue_active(&buf->read_wait) &&
-	    buf->subbufs_produced != buf->subbufs_consumed)
+	    !__stp_relay_buf_empty(buf))
 		wake_up_interruptible(&buf->read_wait);
 }
 
 static void __stp_relay_wakeup_timer(stp_timer_callback_parameter_t unused)
 {
-#ifdef STP_BULKMODE
 	int i;
-#endif
 
-	if (atomic_read(&_stp_relay_data.wakeup)) {
+	if (atomic_cmpxchg(&_stp_relay_data.wakeup, 1, 0)) {
 		struct rchan_buf *buf;
-		
-		atomic_set(&_stp_relay_data.wakeup, 0);
-#ifdef STP_BULKMODE
-		for_each_possible_cpu(i) {
+
+		/* NB it makes no sense to wake up readers on offline CPUs */
+		for_each_online_cpu(i) {
 			buf = _stp_get_rchan_subbuf(_stp_relay_data.rchan->buf,
 						    i);
-			__stp_relay_wakeup_readers(buf);
+
+			/* relay_open() only initializes bufs on the online CPUs
+			 * at the time of invocation. The online CPUs may
+			 * change at any time, so the current online CPU might
+			 * be offline when relay_open() was called.  so we must
+			 * check if the buf for the current CPU is invalid
+			 * otherwise we may dereference a NULL pointer.
+			 */
+			if (likely(buf))
+				__stp_relay_wakeup_readers(buf);
 		}
-#else
-		buf = _stp_get_rchan_subbuf(_stp_relay_data.rchan->buf, 0);
-		__stp_relay_wakeup_readers(buf);
-#endif
 	}
 
 	if (atomic_read(&_stp_relay_data.transport_state) == STP_TRANSPORT_RUNNING)
@@ -167,53 +214,37 @@ static void _stp_transport_data_fs_overwrite(int overwrite)
 	_stp_relay_data.overwrite_flag = overwrite;
 }
 
-#ifdef _STP_USE_DROPPED_FILE
-static int __stp_relay_dropped_open(struct inode *inode, struct file *filp)
-{
-	return 0;
-}
-
-static ssize_t __stp_relay_dropped_read(struct file *filp, char __user *buffer,
-				size_t count, loff_t *ppos)
-{
-	char buf[16];
-
-	snprintf(buf, sizeof(buf), "%u\n",
-		 atomic_read(&_stp_relay_data.dropped));
-
-	return simple_read_from_buffer(buffer, count, ppos, buf, strlen(buf));
-}
-
-static struct file_operations __stp_relay_dropped_fops = {
-	.owner =	THIS_MODULE,
-	.open =		__stp_relay_dropped_open,
-	.read =		__stp_relay_dropped_read,
-};
-#endif
 
 /*
  * Keep track of how many times we encountered a full subbuffer, to aid
  * the user space app in telling how many lost events there were.
  */
 static int __stp_relay_subbuf_start_callback(struct rchan_buf *buf,
-					     void *subbuf, void *prev_subbuf,
-					     size_t prev_padding)
+                                             void *subbuf, void *prev_subbuf,
+                                             size_t prev_padding)
 {
-	if (_stp_relay_data.overwrite_flag || !relay_buf_full(buf))
-		return 1;
-
+        if (_stp_relay_data.overwrite_flag || !relay_buf_full(buf))
+                return 1;
+        
 #ifdef _STP_USE_DROPPED_FILE
-	atomic_inc(&_stp_relay_data.dropped);
+        atomic_inc(&_stp_relay_data.dropped);
 #endif
-	return 0;
+        return 0;
 }
 
+
+// PR26665: demultiplex debugfs vs procfs host
+
 static int __stp_relay_remove_buf_file_callback(struct dentry *dentry)
 {
-	debugfs_remove(dentry);
+        if (debugfs_p)
+                return __stp_debugfs_relay_remove_buf_file_callback(dentry);
+        if (procfs_p)
+                return __stp_procfs_relay_remove_buf_file_callback(dentry);
 	return 0;
 }
 
+
 static struct dentry *
 __stp_relay_create_buf_file_callback(const char *filename,
 				     struct dentry *parent,
@@ -225,39 +256,21 @@ __stp_relay_create_buf_file_callback(const char *filename,
 				     struct rchan_buf *buf,
 				     int *is_global)
 {
-	struct dentry *file = debugfs_create_file(filename, mode, parent, buf,
-	                                          &relay_file_operations_w_owner);
-	/*
-	 * Here's what 'is_global' does (from linux/relay.h):
-	 *
-	 * Setting the is_global outparam to a non-zero value will
-	 * cause relay_open() to create a single global buffer rather
-	 * than the default set of per-cpu buffers.
-	 */
-	if (is_global) {
-#ifdef STP_BULKMODE
-		*is_global = 0;
-#else
-		*is_global = 1;
-#endif
-	}
-
-	if (IS_ERR(file)) {
-		file = NULL;
-	}
-	else if (file) {
-		file->d_inode->i_uid = KUIDT_INIT(_stp_uid);
-		file->d_inode->i_gid = KGIDT_INIT(_stp_gid);
-	}
-	return file;
+        if (debugfs_p)
+                return __stp_debugfs_relay_create_buf_file_callback(filename, parent, mode, buf, is_global);
+        if (procfs_p)
+                return __stp_procfs_relay_create_buf_file_callback(filename, parent, mode, buf, is_global);
+        return NULL;
 }
 
+
 static struct rchan_callbacks __stp_relay_callbacks = {
 	.subbuf_start		= __stp_relay_subbuf_start_callback,
 	.create_buf_file	= __stp_relay_create_buf_file_callback,
 	.remove_buf_file	= __stp_relay_remove_buf_file_callback,
 };
 
+
 static void _stp_transport_data_fs_start(void)
 {
 	if (atomic_read (&_stp_relay_data.transport_state) == STP_TRANSPORT_INITIALIZED) {
@@ -269,6 +282,7 @@ static void _stp_transport_data_fs_start(void)
 
 static void _stp_transport_data_fs_stop(void)
 {
+
 	if (atomic_read (&_stp_relay_data.transport_state) == STP_TRANSPORT_RUNNING) {
 		atomic_set (&_stp_relay_data.transport_state, STP_TRANSPORT_STOPPED);
 		del_timer_sync(&_stp_relay_data.timer);
@@ -281,10 +295,6 @@ static void _stp_transport_data_fs_stop(void)
 static void _stp_transport_data_fs_close(void)
 {
 	_stp_transport_data_fs_stop();
-#ifdef _STP_USE_DROPPED_FILE
-	if (_stp_relay_data.dropped_file)
-		debugfs_remove(_stp_relay_data.dropped_file);
-#endif
 	if (_stp_relay_data.rchan) {
 		relay_close(_stp_relay_data.rchan);
 		_stp_relay_data.rchan = NULL;
@@ -301,33 +311,9 @@ static int _stp_transport_data_fs_init(void)
 	_stp_relay_data.overwrite_flag = 0;
 	_stp_relay_data.rchan = NULL;
 
-#ifdef _STP_USE_DROPPED_FILE
-	atomic_set(&_stp_relay_data.dropped, 0);
-	_stp_relay_data.dropped_file = NULL;
-
-	/* Create "dropped" file. */
-	_stp_relay_data.dropped_file
-		= debugfs_create_file("dropped", 0400, _stp_get_module_dir(),
-				      NULL, &__stp_relay_dropped_fops);
-	if (!_stp_relay_data.dropped_file) {
-		rc = -EIO;
-		goto err;
-	}
-	else if (IS_ERR(_stp_relay_data.dropped_file)) {
-		rc = PTR_ERR(_stp_relay_data.dropped_file);
-		_stp_relay_data.dropped_file = NULL;
-		goto err;
-	}
-
-	_stp_relay_data.dropped_file->d_inode->i_uid = KUIDT_INIT(_stp_uid);
-	_stp_relay_data.dropped_file->d_inode->i_gid = KGIDT_INIT(_stp_gid);
-#endif
-
 	/* Create "trace" file. */
 	npages = _stp_subbuf_size * _stp_nsubbufs;
-#ifdef STP_BULKMODE
 	npages *= num_online_cpus();
-#endif
 	npages >>= PAGE_SHIFT;
 	si_meminfo(&si);
 #define MB(i) (unsigned long)((i) >> (20 - PAGE_SHIFT))
@@ -346,6 +332,8 @@ static int _stp_transport_data_fs_init(void)
 	}
 	relay_file_operations_w_owner = relay_file_operations;
 	relay_file_operations_w_owner.owner = THIS_MODULE;
+	relay_file_operations_w_owner.poll = __stp_relay_file_poll;
+	relay_file_operations_w_owner.read = __stp_relay_file_read;
 #if (RELAYFS_CHANNEL_VERSION >= 7)
 	_stp_relay_data.rchan = relay_open("trace", _stp_get_module_dir(),
 					   _stp_subbuf_size, _stp_nsubbufs,
@@ -357,16 +345,36 @@ static int _stp_transport_data_fs_init(void)
 #endif  /* (RELAYFS_CHANNEL_VERSION < 7) */
 	if (!_stp_relay_data.rchan) {
 		rc = -ENOENT;
+		errk("%s: relay_open() failed: %d\n", THIS_MODULE->name, rc);
 		goto err;
 	}
         /* Increment _stp_allocated_memory and _stp_allocated_net_memory to account for buffers
            allocated by relay_open. */
         {
                 u64 relay_mem;
-                relay_mem = _stp_subbuf_size * _stp_nsubbufs;
-#ifdef STP_BULKMODE
-                relay_mem *= num_online_cpus();
-#endif
+
+		/* relay_open() invokes relay_open_buf() to allocate this part
+		 * of memory for online CPUs only.
+		 */
+		relay_mem = _stp_subbuf_size * _stp_nsubbufs;
+		relay_mem *= num_online_cpus();
+
+		relay_mem += sizeof(struct rchan);
+
+		if (sizeof(_stp_relay_data.rchan->buf) == sizeof(struct rchan_buf *)) {
+			/* newer kernels (newer than 3.10, for example) */
+			/* this part was allocated by alloc_percpu() inside
+			 * relay_open(). */
+			relay_mem += sizeof(struct rchan_buf *) * num_possible_cpus();
+
+		} else {
+			/* for older kernels like 3.10, the buf pointers are
+			 * already part of struct rchan as its array member
+			 * 'struct rchan_buf *buf[NR_CPUS]'. so we do nothing
+			 * here.
+			 */
+		}
+
                 _stp_allocated_net_memory += relay_mem;
                 _stp_allocated_memory += relay_mem;
         }
@@ -404,10 +412,22 @@ _stp_data_write_reserve(size_t size_request, void **entry)
 
 	buf = _stp_get_rchan_subbuf(_stp_relay_data.rchan->buf,
 				    smp_processor_id());
-	if (unlikely(buf->offset + size_request > buf->chan->subbuf_size)) {
+
+	/* relay_open() only initializes bufs on the online CPUs at the time
+	 * of invocation. The online CPUs may change at any time, so the
+	 * current online CPU might be offline when relay_open() was called.
+	 * So we must check if the buf for the current CPU is invalid otherwise
+	 * we may dereference a NULL pointer.
+	 */
+	if (unlikely(buf == NULL))
+		return -EINVAL;
+
+	if (buf->offset >= buf->chan->subbuf_size) {
 		size_request = __stp_relay_switch_subbuf(buf, size_request);
 		if (!size_request)
 			return 0;
+	} else if (buf->offset + size_request > buf->chan->subbuf_size) {
+		size_request = buf->chan->subbuf_size - buf->offset;
 	}
 	*entry = (char*)buf->data + buf->offset;
 	buf->offset += size_request;
@@ -423,6 +443,6 @@ static unsigned char *_stp_data_entry_data(void *entry)
 
 static int _stp_data_write_commit(void *entry)
 {
-	/* Nothing to do here. */
+	atomic_set(&_stp_relay_data.wakeup, 1);
 	return 0;
 }
