@@ -16,9 +16,6 @@ extern "C" {
 #include <stdint.h>
 }
 
-#if __cplusplus >= 201103L /* -std=c++11 */
-#define typeof(x) decltype(x)
-#endif
 
 // The individual probes' info read from the stap module.
 struct dynprobe_location {
