@@ -123,9 +123,12 @@ struct exp_type_details
   // Expand this autocast_op into a function call
   virtual bool expandable() const = 0;
   virtual functioncall *expand(autocast_op* e, bool lvalue) = 0;
+  virtual void print (std::ostream& o) const = 0;
 };
 typedef std::shared_ptr<exp_type_details> exp_type_ptr;
 
+std::ostream& operator << (std::ostream& o, const exp_type_details& d);
+
 
 struct token;
 struct visitor;
@@ -710,6 +713,7 @@ struct statement : public visitable
   statement ();
   statement (const token* tok);
   virtual ~statement ();
+  virtual bool might_pushdown_lock () = 0;
 };
 
 std::ostream& operator << (std::ostream& o, const statement& k);
@@ -726,6 +730,7 @@ struct embeddedcode: public statement
   bool tagged_p (const interned_string& tag) const;
   void print (std::ostream& o) const;
   void visit (visitor* u);
+  virtual bool might_pushdown_lock () { return false; };
 };
 
 
@@ -738,6 +743,7 @@ struct block: public statement
   block (statement* car, statement* cdr);
   block (statement* car, statement* cdr1, statement* cdr2);
   virtual ~block () {}
+  virtual bool might_pushdown_lock () { return true; };
 };
 
 
@@ -748,6 +754,9 @@ struct try_block: public statement
   symbol* catch_error_var; // may be 0
   void print (std::ostream& o) const;
   void visit (visitor* u);
+
+  // PR26296: for try/catch, don't try to push lock/unlock down
+  virtual bool might_pushdown_lock () { return false; };
 };
 
 
@@ -760,6 +769,10 @@ struct for_loop: public statement
   statement* block;
   void print (std::ostream& o) const;
   void visit (visitor* u);
+
+  // PR26269 lockpushdown:
+  // for loops, forget optimizing, just emit locks at top & bottom
+  virtual bool might_pushdown_lock () { return false; };
 };
 
 
@@ -778,6 +791,10 @@ struct foreach_loop: public statement
   statement* block;
   void print (std::ostream& o) const;
   void visit (visitor* u);
+
+  // PR26269 lockpushdown:
+  // for loops, forget optimizing, just emit locks at top & bottom
+  virtual bool might_pushdown_lock () { return false; };
 };
 
 
@@ -786,6 +803,7 @@ struct null_statement: public statement
   void print (std::ostream& o) const;
   void visit (visitor* u);
   null_statement (const token* tok);
+  virtual bool might_pushdown_lock () { return false; };
 };
 
 
@@ -794,6 +812,7 @@ struct expr_statement: public statement
   expression* value;  // executed for side-effects
   void print (std::ostream& o) const;
   void visit (visitor* u);
+  virtual bool might_pushdown_lock () { return false; };
 };
 
 
@@ -804,6 +823,7 @@ struct if_statement: public statement
   statement* elseblock; // may be 0
   void print (std::ostream& o) const;
   void visit (visitor* u);
+  virtual bool might_pushdown_lock () { return true; };
 };
 
 
@@ -811,6 +831,10 @@ struct return_statement: public expr_statement
 {
   void print (std::ostream& o) const;
   void visit (visitor* u);
+
+  // PR26296: We should not encounter a RETURN statement in a
+  // lock-relevant section of code (a probe handler body) at all.
+  virtual bool might_pushdown_lock () { return false; };
 };
 
 
@@ -818,6 +842,7 @@ struct delete_statement: public expr_statement
 {
   void print (std::ostream& o) const;
   void visit (visitor* u);
+  virtual bool might_pushdown_lock () { return false; };
 };
 
 
@@ -825,6 +850,7 @@ struct break_statement: public statement
 {
   void print (std::ostream& o) const;
   void visit (visitor* u);
+  virtual bool might_pushdown_lock () { return false; };
 };
 
 
@@ -832,6 +858,7 @@ struct continue_statement: public statement
 {
   void print (std::ostream& o) const;
   void visit (visitor* u);
+  virtual bool might_pushdown_lock () { return false; };
 };
 
 
@@ -839,6 +866,7 @@ struct next_statement: public statement
 {
   void print (std::ostream& o) const;
   void visit (visitor* u);
+  virtual bool might_pushdown_lock () { return false; };
 };
 
 
@@ -1346,10 +1374,17 @@ struct update_visitor: public visitor
   }
 
   update_visitor(unsigned v = 0): verbose(v), aborted_p(false), relaxed_p(true) {}
-  virtual ~update_visitor() { assert(values.empty()); }
+  virtual ~update_visitor() {
+    if (!values.empty())
+      /* PR27274: tolerate aborted traversals */
+      std::runtime_error(_("update_visitor dtor has unused values"));
+  }
 
   // Permit reuse of the visitor object. 
-  virtual void reset() { aborted_p = false; relaxed_p = true; }
+  virtual void reset() { aborted_p = false; relaxed_p = true;
+    /* PR27274: tolerate aborted traversals */
+    while (!values.empty()) values.pop();
+ }
   virtual bool relaxed() { return relaxed_p; }
     
   virtual void visit_block (block *s);
