@@ -7,6 +7,7 @@
 // Public License (GPL); either version 2, or (at your option) any
 // later version.
 
+#include "config.h"
 #include "session.h"
 #include "tapsets.h"
 #include "translate.h"
@@ -68,7 +69,8 @@ struct be_derived_probe_group: public generic_dpg<be_derived_probe>
   friend bool sort_for_bpf(systemtap_session& s,
 			   be_derived_probe_group *be,
 			   std::vector<derived_probe *> &begin_v,
-			   std::vector<derived_probe *> &end_v);
+			   std::vector<derived_probe *> &end_v,
+                           std::vector<derived_probe *> &error_v);
 
 public:
   void emit_module_decls (systemtap_session& s);
@@ -149,7 +151,7 @@ be_derived_probe_group::emit_module_decls (systemtap_session& s)
 
   s.op->newline() << "static void enter_be_probe (struct stap_be_probe *stp) {";
   s.op->indent(1);
-  common_probe_entryfn_prologue (s, "stp->state", "stp->probe",
+  common_probe_entryfn_prologue (s, "stp->state", "", "stp->probe",
 				 "stp_probe_type_been", false);
   s.op->newline() << "(*stp->probe->ph) (c);";
   common_probe_entryfn_epilogue (s, false, otf_safe_context(s));
@@ -195,7 +197,8 @@ bool
 sort_for_bpf(systemtap_session& s __attribute__ ((unused)),
 	     be_derived_probe_group *be,
 	     std::vector<derived_probe *> &begin_v,
-	     std::vector<derived_probe *> &end_v)
+	     std::vector<derived_probe *> &end_v,
+             std::vector<derived_probe *> &error_v)
 {
   if (!be || be->probes.empty())
     return false;
@@ -207,13 +210,10 @@ sort_for_bpf(systemtap_session& s __attribute__ ((unused)),
       be_derived_probe *p = *i;
       if (p->type == BEGIN)
 	begin_v.push_back(p);
-    }
-
-  for (auto i = be->probes.begin(); i != be->probes.end(); ++i)
-    {
-      be_derived_probe *p = *i;
-      if (p->type != BEGIN)
-	end_v.push_back(p);
+      else if (p->type == END)
+        end_v.push_back(p);
+      else
+        error_v.push_back(p);
     }
 
   return true;
