@@ -7,9 +7,10 @@ void first(){second();}
 
 int main()
 {
+  int rc = 0;
   l1:
     STAP_PROBE(process_by_cmd, main_start);
   first();
   STAP_PROBE(process_by_cmd, main_end);
-  return 0;
+  return rc;
 }
