@@ -14,6 +14,7 @@
 #include <fcntl.h>
 #include <sys/syscall.h>
 #include <limits.h>
+#include <sys/mman.h>
 
 static int sfd;			/* shared between start_server and do_child */
 
@@ -77,6 +78,7 @@ start_server(struct sockaddr_in *sin0)
 
     switch (pid = fork()) {
     case 0:		/* child */
+        alarm(30);
 	do_child();
 	break;
     case -1:			/* fall through */
@@ -98,6 +100,8 @@ int main()
     struct timeval timeout;
     socklen_t fromlen;
 
+    mlockall(MCL_CURRENT);
+
     /* initialize sockaddr's */
     sin1.sin_family = AF_INET;
     sin1.sin_port = htons((getpid() % 32768) + 11000);
@@ -226,9 +230,9 @@ int main()
     recvfrom(s, buf, (size_t)-1, MSG_DONTWAIT, (struct sockaddr *)&from,
 	     &fromlen);
 #if __WORDSIZE == 64
-    //staptest// recvfrom (NNNN, XXXX, 18446744073709551615, MSG_DONTWAIT, XXXX, XXXX) = 6
+    //staptest// recvfrom (NNNN, XXXX, 18446744073709551615, MSG_DONTWAIT, XXXX, XXXX) = NNNN
 #else
-    //staptest// recvfrom (NNNN, XXXX, 4294967295, MSG_DONTWAIT, XXXX, XXXX) = 6
+    //staptest// recvfrom (NNNN, XXXX, 4294967295, MSG_DONTWAIT, XXXX, XXXX) = NNNN
 #endif
 
     close(s);
